@@ -6,7 +6,7 @@ from .client import get_client
 from .utils import to_api, from_api, enum, get_location_id, file_exists
 from .errors import AppPlatformError
 from.generics import AudioMixin
-
+import inspect
 
 class BaseResource(object):
     client = None
@@ -59,18 +59,33 @@ class ListResource(BaseResource):
         """
         raise NotImplemented
 
-    # @classmethod
-    # def as_iterator(cls, chunk=100):
-    #     page = 0
-    #     while True:
-    #         try:
-    #             results = cls.list(page, chunk)
-    #         except Exception:
-    #             break
-    #         # todo: probably break on has-next == False
-    #         page += 1
-    #         for val in results:
-    #             yield val
+    @classmethod
+    def as_iterator(cls, chunk_size=100, **query_params):
+        """
+        Makes a generator that returns chunked pieces of data.
+
+        :param chunk_size: Size of chunk that you want to receive over one iteration.
+                           Default is 100.
+        :param query_params: Keyword arguments that can receive list() method
+        """
+        getargspec_func_list = inspect.getargspec(cls.list)
+
+        if getargspec_func_list.keywords is None:
+            def get_list_results(page):
+                return cls.list(page=page, size=chunk_size)
+        else:
+            def get_list_results(page):
+                return cls.list(page=page, size=chunk_size, **query_params)
+        page = 0
+        while True:
+            results = get_list_results(page)
+
+            if len(results) < chunk_size:
+                yield results
+                break
+            else:
+                yield results
+                page += 1
 
 
 class GenericResource(ListResource, CreateResource):
@@ -363,6 +378,7 @@ class Application(GenericResource):
             cls._path, params=dict(page=page, size=size)).json()
         return [cls(data=from_api(v)) for v in data_as_list]
 
+
     @classmethod
     def get(cls, application_id):
         """
@@ -1322,6 +1338,10 @@ class Media(ListResource):
         data_as_list = client.get(cls._path).json()
         return [cls(data=v) for v in data_as_list]
 
+    @classmethod
+    def as_iterator(cls, chunk_size=100, chunks_amount=5, **query_params):
+        raise NotImplementedError('Media does not support this operation')
+
     def delete(self):
         """
         Deletes a media file from Bandwidth API server. Make sure you don't have any application scripts still
